//
// WtAnimations.js
//
/*global setInterval clearInterval */

protoAnimation = Proto({
	 phase : 0.0,
	 timer : null,
	 speed : 15, // in ms
	 step : 0.1,
	 flags : null,
	 StartAnimationScript : function(){},
	 FinishAnimationScript : function(){},
	 Start : function() {
	  if (this.flags !== null) {
		   // use old flag values
		   // grab steps & step times
		   this.step = 1.0 / (((this.flags >> 21) & 0x0f) + 1);
		   this.speed = 1000/60 * (((this.flags >> 25) & 0x0f) + 1);
	  }
	  this.StartAnimationScript();
	  var self = this;
	  function f() {
		   self.phase += self.step;
		   if (self.phase > 1.0) {
			self.Finish(); // will clear the timer
		   } else {
			self.ShowPhaseScript();
		   }
	  }
	  this.timer = setInterval(f, this.speed);
	  this.phase = 0.0;
	  self.ShowPhaseScript();
	 },
	 Finish : function() {
	  this.phase = 1.0;
	  this.ShowPhaseScript();
	  clearInterval(this.timer);
	  this.timer = null;
	  this.FinishAnimationScript();
	 },
	 ShowPhaseScript : function() {
	  // abstract function to actually do something
	 }
});

protoFadeInAnimation = Proto({
	 _proto : protoAnimation,
	 ShowPhaseScript : function() {
	  this.view.viewElement.style.opacity = this.phase;
	 }
});


protoFadeOutAnimation = Proto({
	_proto : protoAnimation,
	ShowPhaseScript : function() {
		this.view.viewElement.style.opacity = 1.0 - this.phase;
	}
});


function fxSteps(x) { return (x-1) << 21; }
//Sets the number of steps (x) that the animation should take to complete.
//Specify an integer from 1 to 15.
function fxStepTime(x) { return x << 25; }
//Sets the amount of time (x) to take for each
//animation step, in ticks. There are 60 ticks per
//second, or 16.6 milliseconds per tick. Specify an
//integer from 0 to 15.
function fxColumns(x) { return x-1; }
//Sets the number (x) of columns in which to divide
//the view for animation purposes. (ignored)
function fxRows(x) { return (x-1)*32; }
//Sets the number (x) of rows in which to divide the
//view for animation purposes. (ignored)
var fxMoveH =					  0x00010000;
var fxHStartPhase =				  0x00000400;
var fxColAltHPhase =				  0x00001000;
var fxRowAltHPhase =				  0x00004000;
var fxMoveV =					  0x00020000;
var fxVStartPhase =				  0x00000800;
var fxColAltVPhase =				  0x00002000;
var fxRowAltVPhase =				  0x00008000;
var fxLeft =					  0x00010400; //(This flag is the same as specifying fxHStartPhase+fxMoveH.)
var fxRight =					  0x00010000; // (This flag is the same as specifying fxMoveH and not specifying fxHStartPhase.)
var fxUp =					  0x00020800; // (This flag is the same as specifying fxVStartPhase+fxMoveV.)
var fxDown =					  0x00020000; //(This flag is the same as specifying fxMoveV and not specifying fxVStartPhase.)
var fxRevealLine =				 0x00040000; //
var fxWipe =					  0x00080000; // keep in place
var fxFromEdge =				  0x00100000;
//	 steps						   0x00200000;
//	 speed						   0x02000000;
// common effects
var fxCheckerboardEffect =			 0x000260e7;
var fxBarnDoorOpenEffect =			 0x00099401; // fxWipe | fxMoveH | fxRowAltVPhase | fxRowAltHPhase | fxHStartPhase | fxColumns(2)
var fxBarnDoorCloseEffect =			 0x00099001; // fxWipe | fxMoveH | fxRowAltVPhase | fxRowAltHPhase | fxColumns(2)
var fxVenetianBlindsEffect =			 0x000200e0;
var fxIrisOpenEffect =				 0x000f9c21; // fxWipe | fxRevealLine | fxMoveV | fxMoveH | fxRowAltVPhase | fxColAltHPhase | fxHStartPhase | fxVStartPhase
var fxIrisCloseEffect =				 0x000f0c21; // fxWipe | fxRevealLine | fxMoveV | fxMoveH | fxHStartPhase | fxVStartPhase | fxRows(2) | fsColumns(2)
var fxPopDownEffect =				 0x00060000; // fxMoveV | fxRevealLine
var fxDrawerEffect =				  0x00020800;// fxMoveV | fxVStartPhase
var fxZoomOpenEffect =				 0x00039c21;
var fxZoomCloseEffect =				 0x00030c21;
var fxZoomVerticalEffect =			 0x00028820;



protoWipeAnimation = Proto({
	_proto : protoAnimation,
	ClipElement : function(l,t,r,b) {
		// convert from "inset" to clip style
		r = this.view.viewElement.offsetWidth - r;
		b = this.view.viewElement.offsetHeight - b;
		this.view.viewElement.style.clip = "rect(" + t + "px," + r + "px," + b + "px," + l + "px)";
	},
	StartAnimationScript : function() {
		this.startX = parseInt(this.view.viewElement.offsetLeft);
		this.startY = parseInt(this.view.viewElement.offsetTop);
		if (this.direction.indexOf("Out") != -1) {
			this.rephase = function(x) { return 1.0 - x; };
		} else {
			this.rephase = function(x) { return x;};
		}
		if (this.direction.indexOf("Left") != -1) {
			if (this.direction.indexOf("Right") != -1) {
				this.leftScale = 0.5;
				this.rightScale = 0.5;
			} else {
				this.leftScale = 1.0;
				this.rightScale = 0.0;
			}
		} else if (this.direction.indexOf("Right") != -1) {
			this.rightScale = 1.0;
			this.leftScale = 0.0;
		} else {
			this.leftScale = 0.0;
			this.rightScale = 0.0;
		}
		if (this.direction.indexOf("Top") != -1) {
			if (this.direction.indexOf("Bottom") != -1) {
				this.topScale = 0.5;
				this.bottomScale = 0.5;
			} else {
				this.topScale = 1.0;
				this.bottomScale = 0.0;
			}
		} else if (this.direction.indexOf("Bottom") != -1) {
			this.bottomScale = 1.0;
			this.topScale = 0.0;
		} else {
			this.topScale = 0.0;
			this.bottomScale= 0.0;
		}
		this.offsetX = 0.0;
		this.offsetY = 0.0;
		if (this.direction.indexOf("Pan") != -1) {
			// for now
			if (this.direction.indexOf("Left") != -1 || this.direction.indexOf("Right") != -1) {
				this.offsetX = 1.0;
			} else if (this.direction.indexOf("Top") != -1) {
				this.offsetY = -1.0;
			} else if (this.direction.indexOf("Bottom") != -1) {
				this.offsetY = 1.0;
			}
	  }
	 },
	 ShowPhaseScript : function() {
	  var rephase = this.rephase(this.phase);
	  var width = this.view.viewElement.offsetWidth;
	  var height = this.view.viewElement.offsetHeight;
	  var left = rephase * width * this.leftScale;
	  var right = rephase * width * this.rightScale;
	  var top = rephase * height * this.topScale;
	  var bottom = rephase * height * this.bottomScale;
	  //alert('wipe ' + left + ' ' + top + ' ' + right + ' ' + bottom);
	  this.ClipElement(left,top,right,bottom);
	  if (this.offsetX) {
		   this.view.viewElement.style.left = this.offsetX * width * rephase + this.startX;
	  }
	  if (this.offsetY) {
		   this.view.viewElement.style.top = this.offsetY * height * rephase + this.startY;
	  }
	 },
	 FinishAnimationScript : function() {
	  this.view.viewElement.style.clip = null;
	 }
});
